@@ -30,8 +30,6 @@ import widgets
 import classes
 import os
 import os_support
-from sets import Set
-
 from singleton_store import Store, Screen
 
 
@@ -454,12 +452,12 @@ class config_dialog:
 
         # COMMON PROPERTIES
         # LINE COLOR
-        if hasattr( o, 'line_color'):
+        if hasattr( o, 'line_color') and self.line_color.color:
           if self.line_color.color != o.line_color:
             o.line_color = self.line_color.color
             change = 1
         # AREA COLOR
-        if hasattr( o, 'area_color'):
+        if hasattr( o, 'area_color') and self.area_color.color:
           if self.area_color.color != o.area_color:
             o.area_color = self.area_color.color
             change = 1
@@ -532,8 +530,10 @@ class file_properties_dialog:
       t = _('Custom')
     else:
       t = self.paper._paper_properties['type']
+    paper_types = data.paper_types.keys()
+    paper_types.sort()
     self.paper_type_chooser = Pmw.OptionMenu( paper_frame,
-                                              items=data.paper_types.keys(), #+[_('Custom')],
+                                              items=paper_types, #+[_('Custom')],
                                               initialitem = t,
                                               labelpos = 'w',
                                               label_text = _('Paper size')+':',
@@ -673,6 +673,13 @@ class standard_values_dialog:
     self.area_color = widgets.ColorButtonWithTransparencyChecker( color_group.interior(), color=self.standard.area_color, text=_("Area color"))
     self.area_color.pack( side='right', padx=10, pady=5)
 
+    # ATOM
+    atom_group = self.pages.add( _('Atom'))
+    self.show_hydrogens = Tkinter.IntVar()
+    self.show_hydrogens.set( int( self.standard.show_hydrogens))
+    sh = Tkinter.Checkbutton( atom_group, text=_('Show hydrogens on visible atoms'), variable=self.show_hydrogens)
+    sh.pack( anchor='w', padx=10, pady=10)
+
     # BOND
     bond_group = self.pages.add( _("Bond")) #Pmw.Group( self.body, tag_text=_('Bond'))
     # bond width
@@ -697,7 +704,7 @@ class standard_values_dialog:
     self.font_size.pack( anchor = 'nw')
     # font family
     self.font_family = widgets.FontFamilyChooser( font_group, self.standard.font_family)
-    self.font_family.pack( anchor="nw", side = 'bottom')
+    self.font_family.pack( anchor="nw", side='bottom')
 
     # PAPER
     # paper type
@@ -707,8 +714,10 @@ class standard_values_dialog:
       t = _('Custom')
     else:
       t = self.paper._paper_properties['type']
+    paper_types = data.paper_types.keys()
+    paper_types.sort()
     self.paper_type_chooser = Pmw.OptionMenu( paper_group,
-                                              items=data.paper_types.keys(), #+[_('Custom')],
+                                              items=paper_types, #+[_('Custom')],
                                               initialitem = t,
                                               labelpos = 'w',
                                               label_text = _('Paper size')+':',
@@ -806,6 +815,7 @@ class standard_values_dialog:
     st.area_color = self.area_color.color
     st.font_family = self.font_family.getcurselection() and self.font_family.getcurselection()[0] or ''
     st.font_size = int( self.font_size.get())
+    st.show_hydrogens = int( self.show_hydrogens.get())
     # paper properties
     # type
     type = self.paper_type_chooser.getvalue()
@@ -915,7 +925,7 @@ class fragment_dialog( Pmw.Dialog):
                          command=self.done,
                          master='parent')
 
-    self._items = Set()
+    self._items = set()
     self._frags = {}
     self.value = None
     self.init_list()
@@ -964,7 +974,7 @@ class fragment_dialog( Pmw.Dialog):
 
   def clean( self):
     map( self.paper.delete, self._items)
-    self._items = Set()
+    self._items = set()
 
 
   def done( self, button):
@@ -1035,12 +1045,14 @@ class language_dialog( Pmw.Dialog):
   def init_list( self):
     langs = []
     import gettext
-    
+    self.languages = {}
     for lang, language in data.languages.iteritems():
       system = gettext.find( 'BKchem', '/usr/share/locale', [lang])
       local = gettext.find( 'BKchem', os.path.normpath( os.path.join( os_support.get_bkchem_run_dir(), '../locale')), [lang])
       if system or local or lang == "en":
-        langs.append( lang)
+        lang_text = "%s (%s)" % (language, lang)
+        langs.append( lang_text)
+        self.languages[lang_text] = lang
 
     self.list = Pmw.ScrolledListBox( self.interior(),
                                      #selectioncommand=self.select,
