@@ -45,11 +45,8 @@ from temp_manager import template_manager
 import modes
 import interactors
 import os_support
-import popen2
 from id_manager import id_manager
 import string
-from sets import Set
-
 
 import oasa_bridge
 import plugins.plugin
@@ -297,7 +294,7 @@ class BKchem( Tk):
 
     # CREATION OF THE MENU
     
-    menus = Set() # we use this later for plugin entries addition
+    menus = set() # we use this later for plugin entries addition
 
     for temp in self.menu_template:
       if temp[1] == "menu":
@@ -314,7 +311,7 @@ class BKchem( Tk):
         
     # ADDITION OF PLUGINS TO THE MENU
         
-    added_to = Set()
+    added_to = set()
     for name in self.plug_man.get_names( type="script"):
       tooltip = self.plug_man.get_description( name)
       menu = self.plug_man.get_menu( name) 
@@ -562,6 +559,7 @@ class BKchem( Tk):
     dialog = Pmw.MessageDialog(self,
                                title = _('About BKchem'),
                                defaultbutton = 0,
+                               buttons=(_("OK"),),
                                message_text = "BKchem " + _("version") + " " + config.current_BKchem_version + "\n\n" + messages.about_text)
     dialog.iconname('BKchem')
     dialog.activate()
@@ -1289,7 +1287,7 @@ Enter InChI:""")
         #  return
     text = '\n\n'.join( sms)
     dial = Pmw.TextDialog( self,
-                           title='Generated SMILES',
+                           title=_('Generated SMILES'),
                            buttons=(_('OK'),))
     dial.insert( 'end', text)
     dial.activate()
@@ -1393,17 +1391,19 @@ Enter InChI:""")
     try:
       for m in u:
         if m.object_type == 'molecule':
-            inchi, warning = oasa_bridge.mol_to_inchi( m, program)
+            inchi, key, warning = oasa_bridge.mol_to_inchi( m, program)
             sms = sms + warning
-	    sms.append(inchi)
+            sms.append(inchi)
+            sms.append("InChIKey="+key)
+            sms.append("")
     except oasa.oasa_exceptions.oasa_inchi_error, e:
       sms = [_("InChI generation failed,"),_("make sure the path to the InChI program is correct in 'Options/InChI program path'"), "", str( e)]
     except:
       sms = [_("Unknown error occured during InChI generation, sorry."), _("Please, try to make sure the path to the InChI program is correct in 'Options/InChI program path'")]
     self.paper.swap_sides_of_selected("horizontal")
-    text = '\n\n'.join( sms)
+    text = '\n'.join( sms)
     dial = Pmw.TextDialog( self,
-                           title='Generated InChIs',
+                           title=_('Generated InChIs'),
                            buttons=(_('OK'),))
     dial.insert( 'end', text)
     dial.activate()
