@@ -23,9 +23,10 @@
 import misc
 from warnings import warn
 import operator
-import geometry
+from oasa import geometry
 import math
-import transform, transform3d
+from oasa import transform3d
+from oasa import transform
 import time
 import data
 import config
@@ -47,7 +48,7 @@ import oasa
 import Pmw, Tkinter
 from oasa import periodic_table as PT
 import external_data
-from sets import Set
+
 import interactors
 import marks
 import types
@@ -595,8 +596,8 @@ class edit_mode( basic_mode):
     bs = misc.filter_unique( b.atom1.neighbor_edges + b.atom2.neighbor_edges)
     [b.redraw( recalc_side = 1) for b in bs if b.order == 2]
     # all atoms to update
-    as = misc.filter_unique( reduce( operator.add, [[b.atom1,b.atom2] for b in bs], []))
-    [a.reposition_marks() for a in as if isinstance( a, atom)]
+    atms = misc.filter_unique( reduce( operator.add, [[b.atom1,b.atom2] for b in bs], []))
+    [a.reposition_marks() for a in atms if isinstance( a, atom)]
 
 
   def _end_of_empty_drag( self, x1, y1, x2, y2):
@@ -1265,24 +1266,51 @@ class rotate_mode( edit_mode):
     edit_mode.__init__( self)
     self.name = _('rotate')
     self._rotated_mol = None
-    self.submodes = [['2D','3D']] #,['one','all']]
-    self.submodes_names = [[_('2D'),_('3D')]] #,['one object','all objects']]
-    self.submode = [0]
+    self._rotated_atoms = []
+    self.submodes = [['2D','3D'],['normal3d','fixsomething']]
+    self.submodes_names = [[_('2D'),_('3D')],[_('normal 3D rotation'),_('fix selected bond in 3D')]]
+    self.submode = [0,0]
+    self._fixed = None
 
 
   def mouse_down( self, event, modifiers = []):
     edit_mode.mouse_down( self, event, modifiers = modifiers)
     # blocking is not necessary in rotate mode
     self._block_leave_event = 0
+    self._fixed = None
+    self._rotated_atoms = []
+    if self.get_submode(0) == "3D" and self.get_submode(1) == "fixsomething":
+      if len( Store.app.paper.selected) == 1:
+        sel = Store.app.paper.selected[0]
+        if isinstance( sel, bond):
+          self._fixed = sel
+        else:
+          Store.log( _("The selected item must be a bond."), message_type="warn")
+      else:
+        Store.log( _("Exactly one item should be selected to fixed rotation to work, normal rotation will be used."), message_type="hint")
     Store.app.paper.unselect_all()
     if self.focused and (isinstance( self.focused, oasa.graph.vertex) or isinstance(self.focused, bond)):
       self._rotated_mol = self.focused.molecule
+      if self._fixed:
+        # 3D rotation around a bond
+        self._rotated_atoms = self._get_objs_to_rotate()
       x1, y1, x2, y2 = Store.app.paper.list_bbox( [o.item for o in self._rotated_mol.atoms])
       self._centerx = x1+(x2-x1)/2.0
       self._centery = y1+(y2-y1)/2.0
-    if self.focused:
-      self.focused.unfocus()
-      self.focused = None
+    elif self.focused and self.get_submode(0) == '2D' and (isinstance( self.focused, arrow) or (hasattr( self.focused, 'arrow') and isinstance( self.focused.arrow, arrow))):
+      if isinstance( self.focused, arrow):
+        self._rotated_mol = self.focused
+      else:
+        self._rotated_mol = self.focused.arrow
+      x1, y1, x2, y2 = self._rotated_mol.bbox()
+      self._centerx = x1+(x2-x1)/2.0
+      self._centery = y1+(y2-y1)/2.0
+    elif self.focused:
+      if self.get_submode(0) == '3D':
+        tkMessageBox.showerror( _("You can only rotate molecules in 3D!"), _("Sorry but you can only rotate molecules in 3D."))
+      else:
+        tkMessageBox.showerror( _("You can only rotate molecules and arrows in 2D!"), _("Sorry but you can only rotate molecules and arrows in 2D."))
+
     
   def mouse_up( self, event):
     if not self._dragging:
@@ -1296,9 +1324,14 @@ class rotate_mode( edit_mode):
           [a.reposition_marks() for a in self._rotated_mol.atoms]
         self._rotated_mol = None
         Store.app.paper.start_new_undo_record()
+      if self._fixed:
+        Store.app.paper.select( [self._fixed])
     Store.app.paper.add_bindings()
 
   def mouse_drag( self, event):
+    if self.focused:
+      self.focused.unfocus()
+      self.focused = None
     if not self._dragging:
       self._dragging = 1
     if self._rotated_mol:
@@ -1318,22 +1351,61 @@ class rotate_mode( edit_mode):
         self._rotated_mol.transform( tr)
       else:
         # 3D rotation
-        angle1 = round( dx1 / 50.0, 2)
-        angle2 = round( dy1 / 50.0, 2)
-        tr = transform3d.transform3d()
-        tr.set_move( -self._centerx, -self._centery, 0)
-        tr.set_rotation( -angle2, angle1, 0)
-        tr.set_move( self._centerx, self._centery, 0)
-        for a in self._rotated_mol.atoms:
-          x, y, z = a.x, a.y, a.z
-          x, y, z = tr.transform_xyz( x, y, z)
-          a.move_to( x, y)
-          a.z = z
-        for a in self._rotated_mol.bonds:
-          a.simple_redraw()
+        if self.get_submode(1) == "fixsomething" and self._fixed and isinstance( self._fixed, bond):
+          # we have a fixed part
+          if self._fixed.molecule != self._rotated_mol:
+            Store.log( _("You can only rotate the molecule for which you fixed a bond."), message_type="error")
+            return 
+          sig = abs(dx1) > abs(dy1) and misc.signum(dx1) or misc.signum(dy1)
+          angle = round( sig * math.sqrt(dx1**2 +dy1**2) / 50.0, 3)
+          t = geometry.create_transformation_to_rotate_around_particular_axis( self._fixed.atom2.get_xyz(), self._fixed.atom1.get_xyz(), angle)
+          for a in self._rotated_atoms:
+            x, y, z = a.x, a.y, a.z
+            x, y, z = t.transform_xyz( x, y, z)
+            a.move_to( x, y)
+            a.z = z
+          for a in self._rotated_mol.bonds:
+            a.simple_redraw()
+        else:
+          # normal rotation
+          angle1 = round( dx1 / 50.0, 2)
+          angle2 = round( dy1 / 50.0, 2)
+          tr = transform3d.transform3d()
+          tr.set_move( -self._centerx, -self._centery, 0)
+          tr.set_rotation( -angle2, angle1, 0)
+          tr.set_move( self._centerx, self._centery, 0)
+          for a in self._rotated_mol.atoms:
+            x, y, z = a.x, a.y, a.z
+            x, y, z = tr.transform_xyz( x, y, z)
+            a.move_to( x, y)
+            a.z = z
+          for a in self._rotated_mol.bonds:
+            a.simple_redraw()
 
-  def mouse_click( self, even):
-    pass
+  def mouse_click( self, event):
+    edit_mode.mouse_click( self, event)
+
+  def _get_objs_to_rotate( self):
+    if not self._shift:
+      return self._fixed.molecule.atoms
+    b = self._fixed
+    mol = b.molecule
+    mol.temporarily_disconnect_edge( b)
+    cc = list( mol.get_connected_components())
+    mol.reconnect_temporarily_disconnected_edge( b)
+    if len( cc) == 1:
+      return cc[0]
+    else:
+      if isinstance( self.focused, oasa.graph.vertex):
+        to_use = self.focused in cc[0] and cc[0] or cc[1]
+      elif isinstance( self.focused, bond):
+        if self.focused in mol.vertex_subgraph_to_edge_subgraph( cc[0]):
+          to_use = cc[0]
+        else:
+          to_use = cc[1]
+      else:
+        assert isinstance( self.focused, oasa.graph.vertex) or isinstance( self.focused, bond)
+      return to_use
 
 
 class bond_align_mode( edit_mode):
@@ -1991,7 +2063,7 @@ class external_data_mode( basic_mode):
     self.submode = [0,0]
     self.pulldown_menu_submodes = [1]
     self.focused = None
-    self._items = Set()
+    self._items = set()
     self._entries = {}
     self._win = None
     self._active_object = None
@@ -2161,7 +2233,7 @@ class external_data_mode( basic_mode):
   def _delete_table( self, paper=None):
     pap = paper or Store.app.paper
     if self._win:
-      self._items = Set()
+      self._items = set()
       [e.cleanup( pap) for e in self._entries.values()]
       self._entries = {}
       self._frame = None
@@ -2383,25 +2455,76 @@ class bracket_mode( edit_mode):
 class misc_mode( edit_mode):
   """container mode for small, seldom needed modes"""
 
+  wavy_width = 3
+
   def __init__( self):
     edit_mode.__init__( self)
     self.name = _('Miscelanous small modes')
-    self.submodes = [['numbering']]
-    self.submodes_names = [[_('Numbering')]]
+    self.submodes = [['numbering','wavyline']]
+    self.submodes_names = [[_('Numbering'),_("Wavy line")]]
     self.submode = [0]
 
     self._number = 1
-
+    self._line = None
 
 
   def mouse_click( self, event):
-    if self.focused and hasattr( self.focused, 'number'):
-      self.focused.number = str( self._number)
-      self._number += 1
-      for m in self.focused.get_marks_by_type( "atom_number"):
-        m.auto = False
-      
+    if self.get_submode( 0) == "numbering": 
+      if self.focused and hasattr( self.focused, 'number'):
+        self.focused.number = str( self._number)
+        self._number += 1
+        for m in self.focused.get_marks_by_type( "atom_number"):
+          m.auto = False
+        Store.app.paper.start_new_undo_record()
+
+  def mouse_down( self, event, modifiers=None):
+    edit_mode.mouse_down( self, event)
+
+  def mouse_drag( self, event):
+    if self.get_submode( 0) == "numbering":
+      edit_mode.mouse_drag( self, event)
+    else:
+      coords = self._startx, self._starty, event.x, event.y
+      if self._line:
+        Store.app.paper.coords( self._line, *coords)
+      else:
+        self._line = Store.app.paper.create_line( *coords)
+        
+
+  def mouse_up( self, event):
+    if self.get_submode( 0) == "numbering":
+      edit_mode.mouse_up( self, event)
+    elif self.get_submode( 0) == "wavyline":
+      coords = self._startx, self._starty, event.x, event.y
+      if self._line:
+        Store.app.paper.delete( self._line)
+        self._line = None
+        # create the wavy line
+        self._draw_wavy( coords)
       Store.app.paper.start_new_undo_record()
+
+
+  def _draw_wavy( self, coords):
+    x1, y1, x2, y2 = coords
+    # main item
+    x, y, x0, y0 = geometry.find_parallel( x1, y1, x2, y2, self.wavy_width/2.0)
+    d = math.sqrt( (x1-x2)**2 + (y1-y2)**2) # length of the bond
+    step_size = self.wavy_width
+    dx = (x2 - x1)/d 
+    dy = (y2 - y1)/d 
+    ddx = x - x1 
+    ddy = y - y1 
+
+    coords2 = []
+    coords2.extend((x1, y1))
+    for i in range( 0, int( round( d/ step_size))+1):
+      coords = [x1+dx*i*step_size+ddx, y1+dy*i*step_size+ddy, x1+dx*i*step_size-ddx, y1+dy*i*step_size-ddy] 
+      if i % 2:
+        coords2.extend((coords[0], coords[1]))
+      else:
+        coords2.extend((coords[2], coords[3]))
+    coords2.extend((x2, y2)) 
+    Store.app.paper.new_polyline( coords2).draw()
       
 
   def cleanup( self):
@@ -2413,12 +2536,10 @@ class misc_mode( edit_mode):
   def startup( self):
     Store.app.paper.remove_bindings()
     Store.app.paper.add_bindings( active_names=('atom',))
-
     Store.app.paper.unselect_all()
     self._number = 1
     
 
-
   def leave_object( self, event):
     if self.focused:
       self.focused.unfocus()
