@@ -25,7 +25,7 @@ import misc
 import operator
 import copy
 import sys
-
+import transform3d
 
 class cairo_out:
 
@@ -56,13 +56,15 @@ class cairo_out:
                          }
 
   ## taken from: http://bodr.svn.sourceforge.net/viewvc/*checkout*/bodr/trunk/bodr/elements/elements.xml?revision=34&content-type=text%2Fplain
+  ## C and H modified to be black
+  ## F,S,Cl modified to a darker color (suggested by Toon Verstraelen)
   atom_colors_full = {
-    'H': (1.00,1.00,1.00), 'He': (0.85,1.00,1.00),
+    'H': (0.00,0.00,0.00), 'He': (0.85,1.00,1.00),
     'Li': (0.80,0.50,1.00), 'Be': (0.76,1.00,0.00), 'B': (1.00,0.71,0.71),
-    'C': (0.50,0.50,0.50), 'N': (0.05,0.05,1.00), 'O': (1.00,0.05,0.05),
-    'F': (0.70,1.00,1.00), 'Ne': (0.70,0.89,0.96), 'Na': (0.67,0.36,0.95),
+    'C': (0.0,0.0,0.0), 'N': (0.05,0.05,1.00), 'O': (1.00,0.05,0.05),
+    'F': (0.30,0.70,0.70), 'Ne': (0.70,0.89,0.96), 'Na': (0.67,0.36,0.95),
     'Mg': (0.54,1.00,0.00), 'Al': (0.75,0.65,0.65), 'Si': (0.50,0.60,0.60),
-    'P': (1.00,0.50,0.00), 'S': (1.00,1.00,0.19), 'Cl': (0.12,0.94,0.12),
+    'P': (1.00,0.50,0.00), 'S': (0.80,0.80,0.20), 'Cl': (0.00,0.80,0.00),
     'Ar': (0.50,0.82,0.89), 'K': (0.56,0.25,0.83), 'Ca': (0.24,1.00,0.00),
     'Sc': (0.90,0.90,0.90), 'Ti': (0.75,0.76,0.78), 'V': (0.65,0.65,0.67),
     'Cr': (0.54,0.60,0.78), 'Mn': (0.61,0.48,0.78), 'Fe': (0.50,0.48,0.78),
@@ -106,7 +108,9 @@ class cairo_out:
     # should atom coordinates be rounded to whole pixels before rendering?
     # This improves image sharpness but might slightly change the geometry
     'align_coords': True,
+    # the following also draws hydrogens on shown carbons
     'show_hydrogens_on_hetero': False,
+    'show_carbon_symbol': False, 
     'margin': 15,
     'line_width': 1.0,
     # how far second bond is drawn
@@ -114,7 +118,8 @@ class cairo_out:
     'wedge_width': 6.0,
     'font_name': "Arial",
     'font_size': 16,
-    'background_color': (1,1,1),
+    # background color in RGBA
+    'background_color': (1,1,1,1),
     'color_atoms': True,
     'color_bonds': True,
     'space_around_atom': 2,
@@ -156,66 +161,74 @@ class cairo_out:
     for e in copy.copy( mol.edges):
       self._draw_edge( e)
 
-  def create_surface( self, w, h):
+  def create_surface( self, w, h, format):
     """currently implements PNG writting, but might be overriden to write other types;
     w and h are minimal estimated width and height"""
     # trick - we use bigger surface and then copy from it to a new surface and crop
-    self.surface = cairo.ImageSurface( cairo.FORMAT_ARGB32, w, h)
+    if format == "png":
+      self.surface = cairo.ImageSurface( cairo.FORMAT_ARGB32, w, h)
+    elif format == "pdf":
+      self.surface = cairo.PDFSurface( self.filename, w, h)
+    elif format == "svg":
+      self.surface = cairo.SVGSurface( self.filename, w, h)
+    else:
+      raise Exception( "unknown format '%s'" % format)
 
+  def init_surface( self):
+    """make all necessary operations to prepare a surface for drawing:
+    set antialiasing as requested, create background, etc."""
+    pass
+
+  def create_dummy_surface( self, w, h):
+    self.surface = cairo.ImageSurface( cairo.FORMAT_A1, w, h)
+    
 
   def write_surface( self):
-    """currently implements PNG writting, but might be overriden to write other types"""
-    # Because it is not possible to calculate the bounding box of a drawing before its drawn (mainly
-    # because we don't know the size of text items), this object internally draws to a surface with
-    # large margins and the saves a cropped version into a file (this is done by drawing to a new
-    # surface and using the old one as source.
-    #self.surface.write_to_png( self.filename)
-    #return 
-    # real width and height
-    x1, y1, x2, y2 = self._get_bbox()
-    x1, y1 = self.context.user_to_device( x1, y1)
-    x2, y2 = self.context.user_to_device( x2, y2)
-    width = int( x2 - x1 + 2*self.margin*self.scaling)
-    height = int( y2 - y1 + 2*self.margin*self.scaling)
-    surface = cairo.ImageSurface( cairo.FORMAT_ARGB32, width, height)
-    context = cairo.Context( surface)
-    context.set_source_rgb( *self.background_color)
-    context.rectangle( 0, 0, width, height)
-    context.fill()
-    context.set_source_surface( self.surface, round( -x1+self.margin*self.scaling), round( -y1+self.margin*self.scaling))
-    context.rectangle( 0, 0, width, height)
-    context.fill()
-    context.show_page()
-    surface.write_to_png( self.filename)
-    surface.finish()
+    """finishes the surface and write it to the file if necessary"""
+    self.context.show_page()
+    if isinstance( self.surface, cairo.ImageSurface):
+      self.surface.write_to_png( self.filename)
+    self.surface.finish()
 
 
-  def mol_to_cairo( self, mol, filename):
+  def mols_to_cairo( self, mols, filename, format="png"):
     x1, y1, x2, y2 = None, None, None, None
-    for v in mol.vertices:
-      v.y = -v.y # flip coords - molfiles have them the other way around
-      if self.align_coords:
-        v.x = self._round( v.x)
-        v.y = self._round( v.y)
-      if x1 == None or x1 > v.x:
-        x1 = v.x
-      if x2 == None or x2 < v.x:
-        x2 = v.x
-      if y1 == None or y1 > v.y:
-        y1 = v.y
-      if y2 == None or y2 < v.y:
-        y2 = v.y
+    for mol in mols:
+      for v in mol.vertices:
+        v.y = -v.y # flip coords - molfiles have them the other way around
+        if self.align_coords:
+          v.x = self._round( v.x)
+          v.y = self._round( v.y)
+        if x1 == None or x1 > v.x:
+          x1 = v.x
+        if x2 == None or x2 < v.x:
+          x2 = v.x
+        if y1 == None or y1 > v.y:
+          y1 = v.y
+        if y2 == None or y2 < v.y:
+          y2 = v.y
     w = int( x2 - x1)
     h = int( y2 - y1)
     self._bboxes.append( (x1,y1,x2,y2))
 
+    # dummy surface to get complete size of the drawing
+    # Because it is not possible to calculate the bounding box of a drawing before its drawn (mainly
+    # because we don't know the size of text items), this object internally draws to a surface with
+    # large margins to get the bbox
+    _w = int( self.scaling*w+2*self.scaling*self._temp_margin)
+    _h = int( self.scaling*h+2*self.scaling*self._temp_margin)
+    self.create_dummy_surface( _w, _h)
+    self.context = cairo.Context( self.surface)
+    [self.draw_mol( mol) for mol in mols]
+    x1, y1, x2, y2 = self._get_bbox()
+    x1, y1 = self.context.user_to_device( x1, y1)
+    x2, y2 = self.context.user_to_device( x2, y2)
+    width = int( self.scaling*(x2-x1) + 2*self.margin*self.scaling)
+    height = int( self.scaling*(y2-y1) + 2*self.margin*self.scaling)
+
+    # now paint for real
     self.filename = filename
-
-    # create cairo surface for drawing
-    _w = int( w+2*self.scaling*self._temp_margin)
-    _h = int( h+2*self.scaling*self._temp_margin)
-    self.create_surface( _w, _h)
-
+    self.create_surface( width, height, format)
     self.context = cairo.Context( self.surface)
     if not self.antialias_drawing:
       self.context.set_antialias( cairo.ANTIALIAS_NONE)
@@ -223,19 +236,25 @@ class cairo_out:
       options = self.context.get_font_options()
       options.set_antialias( cairo.ANTIALIAS_NONE)
       self.context.set_font_options( options)
-    self.context.translate( round( -x1*self.scaling+self.scaling*self._temp_margin), round( -y1*self.scaling+self.scaling*self._temp_margin))
+    self.context.translate( round( -x1*self.scaling+self.scaling*self.margin), round( -y1*self.scaling+self.scaling*self.margin))
     self.context.scale( self.scaling, self.scaling)
     self.context.rectangle( x1, y1, w, h)
+    self._set_source_color( self.background_color)
+    self.context.paint()
     self.context.new_path()
     self.context.set_source_rgb( 0, 0, 0)
-    self.draw_mol( mol)
-    self.context.show_page()
+    [self.draw_mol( mol) for mol in mols]
     # write the content to the file
     self.write_surface()
-    self.surface.finish()
     # flip y coordinates back
     for v in mol.vertices:
       v.y = -v.y
+    
+
+  def mol_to_cairo( self, mol, filename, format="png"):
+    """This is a convenience method kept for backward compatibility,
+    it just calls mols_to_cairo internally"""
+    return self.mols_to_cairo( [mol], filename, format=format)
 
   def _round( self, x):
     if self.line_width % 2:
@@ -320,85 +339,121 @@ class cairo_out:
       self.context.stroke()
 
     # code itself
-
+    # at first detect the need to make 3D adjustments
+    self._transform = transform3d.transform3d()
+    self._invtransform = transform3d.transform3d()
+    transform = None
+    if e.order > 1:
+      atom1,atom2 = e.vertices
+      for n in atom1.neighbors + atom2.neighbors:
+        # e.atom1 and e.atom2 are in this list as well
+        if n.z != 0:
+          # engage 3d transform prior to detection of where to draw
+          transform = self._get_3dtransform_for_drawing( e)
+          #transform = None
+          break
+      if transform:
+        for n in atom1.neighbors + atom2.neighbors:
+          n.coords = transform.transform_xyz( *n.coords)
+        self._transform = transform
+        self._invtransform = transform.get_inverse()
+    # // end of 3D adjustments
+    # now the code itself
     coords = self._where_to_draw_from_and_to( e)
-    if not coords:
-      return 
-    start = coords[:2]
-    end = coords[2:]
-    v1, v2 = e.vertices
-    if self.color_bonds:
-      color1 = self.atom_colors.get( v1.symbol, (0,0,0))
-      color2 = self.atom_colors.get( v2.symbol, (0,0,0))
-    else:
-      color1 = color2 = (0,0,0)
-    has_shown_vertex = bool( [1 for _v in e.vertices if _v in self._vertex_to_bbox])
-
-    if e.order == 1:
-      if e.type == 'w':
-        draw_plain_or_colored_wedge( start, end)
-      elif e.type == 'h':
-        draw_plain_or_colored_hatch( start, end)
+    if coords:
+      start = coords[:2]
+      end = coords[2:]
+      v1, v2 = e.vertices
+      if self.color_bonds:
+        color1 = self.atom_colors.get( v1.symbol, (0,0,0))
+        color2 = self.atom_colors.get( v2.symbol, (0,0,0))
       else:
-        draw_plain_or_colored_line( start, end)
-      
-    if e.order == 2:
-      side = 0
-      # find how to center the bonds
-      # rings have higher priority in setting the positioning
-      in_ring = False
-      for ring in self.molecule.get_smallest_independent_cycles():
-        double_bonds = len( [b for b in self.molecule.vertex_subgraph_to_edge_subgraph(ring) if b.order == 2])
-        if v1 in ring and v2 in ring:
-          in_ring = True
-          side += double_bonds * reduce( operator.add, [geometry.on_which_side_is_point( start+end, (a.x,a.y)) for a in ring if a!=v1 and a!=v2])
-      # if rings did not decide, use the other neigbors
-      if not side:
-        for v in v1.neighbors + v2.neighbors:
-          if v != v1 and v!= v2:
-            side += geometry.on_which_side_is_point( start+end, (v.x, v.y))
-      # if neighbors did not decide either
-      if not side and (in_ring or not has_shown_vertex):
-        if in_ring:
-          # we don't want centered bonds inside rings
-          side = 1 # select arbitrary value
+        color1 = color2 = (0,0,0)
+      has_shown_vertex = bool( [1 for _v in e.vertices if _v in self._vertex_to_bbox])
+
+      if e.order == 1:
+        if e.type == 'w':
+          draw_plain_or_colored_wedge( start, end)
+        elif e.type == 'h':
+          draw_plain_or_colored_hatch( start, end)
         else:
-          # bond between two unshown atoms - we want to center them only in some cases
-          if len( v1.neighbors) < 3 and len( v2.neighbors) < 3:
-            # try to figure out which side is more towards the center of the molecule
-            side = reduce( operator.add, [geometry.on_which_side_is_point( start+end, (a.x,a.y))
-                                          for a in self.molecule.vertices if a!=v1 and a!=v2])
-            if not side:
-              side = 1 # we choose arbitrary value, we don't want centering
-      if side:
-        draw_plain_or_colored_line( start, end)
-        x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], self.bond_width*misc.signum( side))
-        # shorten the second line
-        length = geometry.point_distance( x1,y1,x2,y2)
-        if v2 not in self._vertex_to_bbox:
-          x2, y2 = geometry.elongate_line( x1, y1, x2, y2, -self.bond_second_line_shortening*length)
-        if v1 not in self._vertex_to_bbox:
-          x1, y1 = geometry.elongate_line( x2, y2, x1, y1, -self.bond_second_line_shortening*length)
-        draw_plain_or_colored_line( (x1, y1), (x2, y2), second=True)
-      else:
-        for i in (1,-1):
-          x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], i*self.bond_width*0.5)
-          draw_plain_or_colored_line( (x1, y1), (x2, y2))
+          draw_plain_or_colored_line( start, end)
 
-    elif e.order == 3:
-      self._draw_line( start, end, line_width=self.line_width)
-      for i in (1,-1):
-        x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], i*self.bond_width*0.7)
-        draw_plain_or_colored_line( (x1, y1), (x2, y2), second=True)
+      if e.order == 2:
+        side = 0
+        # find how to center the bonds
+        # rings have higher priority in setting the positioning
+        in_ring = False
+        for ring in self.molecule.get_smallest_independent_cycles():
+          double_bonds = len( [b for b in self.molecule.vertex_subgraph_to_edge_subgraph(ring) if b.order == 2])
+          if v1 in ring and v2 in ring:
+            in_ring = True
+            side += double_bonds * reduce( operator.add, [geometry.on_which_side_is_point( start+end, (a.x,a.y)) for a in ring if a!=v1 and a!=v2])
+        # if rings did not decide, use the other neigbors
+        if not side:
+          for v in v1.neighbors + v2.neighbors:
+            if v != v1 and v!= v2:
+              side += geometry.on_which_side_is_point( start+end, (v.x, v.y))
+        # if neighbors did not decide either
+        if not side and (in_ring or not has_shown_vertex):
+          if in_ring:
+            # we don't want centered bonds inside rings
+            side = 1 # select arbitrary value
+          else:
+            # bond between two unshown atoms - we want to center them only in some cases
+            if len( v1.neighbors) == 1 and len( v2.neighbors) == 1:
+              # both atoms have only one neighbor 
+              side = 0
+            elif len( v1.neighbors) < 3 and len( v2.neighbors) < 3:
+              # try to figure out which side is more towards the center of the molecule
+              side = reduce( operator.add, [geometry.on_which_side_is_point( start+end, (a.x,a.y))
+                                            for a in self.molecule.vertices if a!=v1 and a!=v2], 0)
+              if not side:
+                side = 1 # we choose arbitrary value, we don't want centering
+        if side:
+          draw_plain_or_colored_line( start, end)
+          x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], self.bond_width*misc.signum( side))
+          # shorten the second line
+          length = geometry.point_distance( x1,y1,x2,y2)
+          if v2 not in self._vertex_to_bbox:
+            x2, y2 = geometry.elongate_line( x1, y1, x2, y2, -self.bond_second_line_shortening*length)
+          if v1 not in self._vertex_to_bbox:
+            x1, y1 = geometry.elongate_line( x2, y2, x1, y1, -self.bond_second_line_shortening*length)
+          draw_plain_or_colored_line( (x1, y1), (x2, y2), second=True)
+        else:
+          for i in (1,-1):
+            x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], i*self.bond_width*0.5)
+            draw_plain_or_colored_line( (x1, y1), (x2, y2))
+
+      elif e.order == 3:
+        self._draw_line( start, end, line_width=self.line_width)
+        for i in (1,-1):
+          x1, y1, x2, y2 = geometry.find_parallel( start[0], start[1], end[0], end[1], i*self.bond_width*0.7)
+          draw_plain_or_colored_line( (x1, y1), (x2, y2), second=True)
+
+    if transform:
+      # if transform was used, we need to transform back
+      for n in atom1.neighbors + atom2.neighbors:
+        n.coords = self._invtransform.transform_xyz( *n.coords)
     
 
   def _where_to_draw_from_and_to( self, b):
+    def fix_bbox( a):
+      x, y = a.x, a.y
+      data = self._vertex_to_bbox.get( a, None)
+      if data:
+        (ox, oy), bbox = data
+        dx = x - ox
+        dy = y - oy
+        bbox = [bbox[0]+dx,bbox[1]+dy,bbox[2]+dx,bbox[3]+dy]
+        return bbox
+      return None
     # at first check if the bboxes are not overlapping
     atom1, atom2 = b.vertices
     x1, y1 = atom1.x, atom1.y
     x2, y2 = atom2.x, atom2.y
-    bbox1 = self._vertex_to_bbox.get( atom1, None)
-    bbox2 = self._vertex_to_bbox.get( atom2, None)
+    bbox1 = fix_bbox( atom1)
+    bbox2 = fix_bbox( atom2)
     if bbox1 and bbox2 and geometry.do_rectangles_intersect( bbox1, bbox2):
       return None
     # then we continue with computation
@@ -436,12 +491,16 @@ class cairo_out:
 
   def _draw_vertex( self, v):
     pos = sum( [(a.x < v.x) and -1 or 1 for a in v.neighbors if abs(a.x-v.x)>0.2])
-    if v.symbol != "C":
+    if 'show_symbol' in v.properties_:
+      show_symbol = v.properties_['show_symbol']
+    else:
+      show_symbol = (v.symbol != "C" or v.degree == 0 or self.show_carbon_symbol)
+    if show_symbol:
       x = v.x
       y = v.y
       text = v.symbol
       hs = ""
-      if self.show_hydrogens_on_hetero:
+      if self.show_hydrogens_on_hetero or v.properties_.get( 'show_hydrogens', False):
         if v.free_valency == 1:
           hs = "H"
         elif v.free_valency > 1:
@@ -476,7 +535,7 @@ class cairo_out:
         color = (0,0,0)
       center_letter = pos <= 0 and 'first' or 'last'
       bbox = self._draw_text( (x,y), text, center_letter=center_letter, color=color)
-      self._vertex_to_bbox[v] = geometry.expand_rectangle( bbox, self.space_around_atom)
+      self._vertex_to_bbox[v] = ((x,y), geometry.expand_rectangle( bbox, self.space_around_atom))
 
       # sometimes charge is done here, if it wasn't done before
       if charge:
@@ -505,6 +564,28 @@ class cairo_out:
         self.context.move_to( round( x-xbearing-0.5*width), round( y+0.5*height))
         self.context.show_text( charge)
 
+  def _get_3dtransform_for_drawing( self, b):
+    """this is a helper method that returns a transform3d which rotates
+    a bond and its neighbors to coincide with the x-axis and rotates neighbors to be in (x,y)
+    plane."""
+    atom1, atom2 = b.vertices
+    x1,y1,z1 = atom1.coords
+    x2,y2,z2 = atom2.coords
+    t = geometry.create_transformation_to_coincide_point_with_z_axis( [x1,y1,z1],[x2,y2,z2])
+    x,y,z = t.transform_xyz( x2,y2,z2)
+    # now rotate to make the plane of neighbor atoms coincide with x,y plane
+    angs = []
+    for n in atom1.neighbors + atom2.neighbors:
+      if n is not atom1 and n is not atom2:
+        nx,ny,nz = t.transform_xyz( *n.coords)
+        ang = math.atan2( ny, nx)
+        if ang < -0.00001:
+          ang += math.pi
+        angs.append( ang)
+    ang = sum( angs) / len( angs)
+    t.set_rotation_z( ang + math.pi/2.0)
+    t.set_rotation_y( math.pi/2.0)
+    return t
 
 
   ## ------------------------------ lowlevel drawing methods ------------------------------
@@ -542,11 +623,10 @@ class cairo_out:
   def _draw_text( self, xy, text, font_name=None, font_size=None, center_letter=None,
                   color=(0,0,0)):
     import xml.sax
-    from sets import Set
     class text_chunk:
       def __init__( self, text, attrs=None):
         self.text = text
-        self.attrs = attrs or Set()
+        self.attrs = attrs or set()
 
     class FtextHandler ( xml.sax.ContentHandler):
       def __init__( self):
@@ -562,7 +642,7 @@ class cairo_out:
         self._above.pop( -1)
       def _closeCurrentText( self):
         if self._text:
-          self.chunks.append( text_chunk( self._text, attrs = Set( self._above)))
+          self.chunks.append( text_chunk( self._text, attrs = set( self._above)))
           self._text = ""
       def characters( self, data):
         self._text += data
@@ -626,7 +706,7 @@ class cairo_out:
       # background
       if self.add_background_to_text:
         self.context.rectangle( x1+xbearing, y1+ybearing, width, height)
-        self.context.set_source_rgb( *self.background_color)
+        self._set_source_color( self.background_color)
         self.context.fill()
         #self.context.set_line_width( 3)
         #self.context.stroke()
@@ -644,7 +724,7 @@ class cairo_out:
     return bbox
 
 
-
+  # not used
   def _draw_rectangle( self, coords, fill_color=(1,1,1)):
     #outline = self.paper.itemcget( item, 'outline')
     x1, y1, x2, y2 = coords
@@ -659,6 +739,7 @@ class cairo_out:
     
     
   def _create_cairo_path( self, points, closed=False):
+    points = [self._invtransform.transform_xyz( p[0],p[1],0)[:2] for p in points]
     x, y = points[0]
     self.context.move_to( x, y)
     for (x,y) in points[1:]:
@@ -680,21 +761,46 @@ class cairo_out:
         bbox[3] = y2
     return bbox
 
+  def _set_source_color( self, color):
+    """depending on the value of color uses the proper method,
+    either set_source_rgb or set_source_rgba"""
+    if len( color) == 3:
+      self.context.set_source_rgb( *color)
+    elif len( color) == 4:
+      self.context.set_source_rgba( *color)
+    else:
+      raise ValueError( "wrong specification of color '%s'" % color)
+    
 
-def mol_to_png( mol, filename):
-  c = cairo_out()
+def mol_to_png( mol, filename, **kw):
+  c = cairo_out( **kw)
   c.mol_to_cairo( mol, filename)
 
+def mols_to_png( mols, filename, **kw):
+  c = cairo_out( **kw)
+  c.mols_to_cairo( mols, filename)
+
+def mol_to_cairo( mol, filename, format, **kw):
+  c = cairo_out( **kw)
+  c.mol_to_cairo( mol, filename, format=format)
+
+def mols_to_cairo( mols, filename, format, **kw):
+  c = cairo_out( **kw)
+  c.mols_to_cairo( mols, filename, format=format)
+
 
 if __name__ == "__main__":
 
-##   import smiles
+  import smiles
 
-##   mol = smiles.text_to_mol( "c1cc(C#N)ccc1N", calc_coords=30)
+  mol = smiles.text_to_mol( "FCCSCl", calc_coords=30)
+  #mol.vertices[0].properties_['show_hydrogens'] = False
+  #mol.vertices[1].properties_['show_symbol'] = False
+  #mol.vertices[2].properties_['show_symbol'] = True
+  mol_to_png( mol, "output.png", show_hydrogens_on_hetero=True, scaling=2)
+
+##   import inchi
+##   mol = inchi.text_to_mol( "1/C7H6O2/c8-7(9)6-4-2-1-3-5-6/h1-5H,(H,8,9)", include_hydrogens=False, calc_coords=30)
 ##   mol_to_png( mol, "output.png")
 
-  import inchi
-  mol = inchi.text_to_mol( "1/C7H6O2/c8-7(9)6-4-2-1-3-5-6/h1-5H,(H,8,9)", include_hydrogens=False, calc_coords=30)
-  mol_to_png( mol, "output.png")
-
 
