@@ -31,7 +31,7 @@ import math
 import operator
 
 from singleton_store import Screen
-import transform3d
+from oasa import transform3d
 
 
 def read_smiles( text, paper):
@@ -55,13 +55,18 @@ def read_inchi( text, paper):
   
 def mol_to_inchi( mol, program):
   m = bkchem_mol_to_oasa_mol( mol)
-  # we do not use mol_to_text because generate_inchi returns extra warning messages
-  return oasa.inchi.generate_inchi( m, program=program)
+  # we do not use mol_to_text because generate_inchi_and_inchikey returns extra warning messages
+  _inchi, _key, _warnings = oasa.inchi.generate_inchi_and_inchikey( m, program=program, fixed_hs=False)
+  return (_inchi, _key, _warnings)
 
 
 def read_molfile( file, paper):
   mol = oasa.molfile.file_to_mol( file)
-  return oasa_mol_to_bkchem_mol( mol, paper)
+  if not mol.is_connected():
+    mols = mol.get_disconnected_subgraphs()
+    return [oasa_mol_to_bkchem_mol( mol, paper) for mol in mols]
+  else:
+    return [oasa_mol_to_bkchem_mol( mol, paper)]
 
 
 def write_molfile( mol, file):
