@@ -22,13 +22,13 @@
 
 from __future__ import division
 
-from Tkinter import Canvas
+from Tkinter import Canvas, ALL
 import tkFont, tkMessageBox
 import classes
 import arrow
 from molecule import molecule
-from transform import transform 
-from transform3d import transform3d
+from oasa.oasa.transform import transform 
+from oasa.oasa.transform3d import transform3d
 import misc
 from temp_manager import template_manager
 import string
@@ -60,8 +60,7 @@ from group import group
 from atom import atom
 from textatom import textatom
 import oasa
-import geometry
-from sets import Set
+from oasa import geometry
 from id_manager import id_manager
 import interactors
 import exceptions
@@ -209,7 +208,7 @@ class chem_paper( Canvas, object):
   selected_atoms = property( _get_selected_atoms)
 
   def _get_selected_bonds( self):
-    return [o for o in self.selected if isinstance( o, bond)]
+    return [o for o in self.selected if isinstance( o, oasa.graph.edge)]
 
   selected_bonds = property( _get_selected_bonds)
 
@@ -554,7 +553,7 @@ class chem_paper( Canvas, object):
 
 
   def bonds_to_update( self, exclude_selected_bonds=True):
-    a = reduce( operator.or_, map( Set, [v.get_neighbor_edges() for v in self.selected if v.object_type == "atom"]), Set())
+    a = reduce( operator.or_, map( set, [v.get_neighbor_edges() for v in self.selected if v.object_type == "atom"]), set())
     # if bond is also selected then it moves with and should not be updated
     if exclude_selected_bonds:
       return [b for b in a if b not in self.selected]
@@ -1233,7 +1232,19 @@ class chem_paper( Canvas, object):
     # locate all selected top_levels, filter them to be unique
     cp, unique = self.selected_to_unique_top_levels()
     # now find center of bbox of all objects in cp
-    xmin, ymin, xmax, ymax = self.common_bbox( cp)
+    self.place_next_to_bbox( mode, align, dist, obj, self.common_bbox( cp))
+    self.start_new_undo_record()
+
+
+
+  def place_next_to_bbox( self, mode, align, dist, obj, bbox):
+    """Places an object (obj) in a distance (dist) next to the bbox,
+    by changing the x or the y value of the object according to the mode.
+    Modes: l= left r=right a=above b=below
+    Align: t=top b=bottom l=left r=right h=horizontal v=vertical
+    align or mode can be set to "" to use only one function"""
+    # now find center of bbox of all objects in cp
+    xmin, ymin, xmax, ymax = bbox
     x1o,y1o,x2o,y2o = obj.bbox()
     if mode == "l":
         obj.move(xmin-x2o-dist, 0)
@@ -1255,7 +1266,6 @@ class chem_paper( Canvas, object):
         obj.move ((xmax+xmin)/2-(x1o+x2o)/2,0)
     elif align == "h":
         obj.move (0,(ymax+ymin)/2-(y1o+y2o)/2)
-    self.start_new_undo_record()
 
 
 
@@ -1412,7 +1422,7 @@ class chem_paper( Canvas, object):
     if not self.changes_made:
       self.changes_made = 1
     self.um.start_new_record( name=name)
-
+    self.after_undo_record()
 
 
   def before_undo_record( self):
@@ -1420,6 +1430,16 @@ class chem_paper( Canvas, object):
     undo is recorded should be done"""
     checks.check_linear_fragments( self)
 
+  def after_undo_record( self):
+    """similar to before_undo_record but is run after the undo was recorded"""
+    # check the bbox to see if we need to update scroll region
+    if not hasattr( self, "_old_bbox"):
+      self._old_bbox = self.bbox(ALL)
+      self.update_scrollregion()
+    else:
+      _bbox = self.bbox(ALL)
+      if _bbox != self._old_bbox:
+        self.update_scrollregion()
 
 
   def display_weight_of_selected( self):
@@ -1452,6 +1472,8 @@ class chem_paper( Canvas, object):
       dialog.insert( 'end', "\n")
       dialog.insert( 'end', _("Weight: %4.3f") % comp.get_molecular_weight())
       dialog.insert( 'end', "\n")
+      dialog.insert( 'end', _("Monoisotopic mass: %12.8f") % comp.get_exact_molecular_mass())
+      dialog.insert( 'end', "\n")
       dialog.insert( 'end', _("Composition: %s") % PT.dict_to_composition( comp))
       dialog.insert( 'end', "\n\n")
     if len( s_mols) > 1:
@@ -1465,6 +1487,8 @@ class chem_paper( Canvas, object):
       dialog.insert( 'end', "\n")
       dialog.insert( "end", _("Weight: %4.3f") % comps.get_molecular_weight())
       dialog.insert( 'end', "\n")
+      dialog.insert( 'end', _("Monoisotopic mass: %12.8f") % comps.get_exact_molecular_mass())
+      dialog.insert( 'end', "\n")
       dialog.insert( 'end', _("Composition: %s") % PT.dict_to_composition( comps))
     dialog.tag_config( 'headline', underline=1)
     dialog.activate()
@@ -1664,7 +1688,7 @@ class chem_paper( Canvas, object):
     self._paper_properties['crop_margin'] = self.standard.paper_crop_margin
     self._paper_properties['use_real_minus'] = Store.pm.get_preference( "use_real_minus") or 0
     self._paper_properties['replace_minus'] = Store.pm.get_preference( "replace_minus") or 0
-
+    self.update_scrollregion()
 
     
 
@@ -1717,8 +1741,12 @@ class chem_paper( Canvas, object):
       self._paper_properties['replace_minus'] = replace_minus
 
     self.create_background()
+    self.update_scrollregion()
 
 
+  def update_scrollregion( self):
+    x1,y1,x2,y2 = self.bbox(ALL)
+    self.config( scrollregion=(x1-100,y1-100,x2+100,y2+100))
 
 
 
@@ -1776,7 +1804,7 @@ class chem_paper( Canvas, object):
         for a in m.atoms:
           a.read_standard_values( self.standard, old_standard=old_standard)
           to_redraw.append( a)
-      else:
+      elif m.object_type != "point":
         m.read_standard_values( self.standard, old_standard=old_standard)
         to_redraw.append( m)
     return to_redraw
@@ -1947,8 +1975,8 @@ class chem_paper( Canvas, object):
     mols, u = self.selected_to_unique_top_levels()
     for mol in mols:
       if isinstance( mol, molecule):
-        notselected = Set( mol.atoms) - Set( self.selected)
-        selected = Set( mol.atoms) & Set( self.selected)
+        notselected = set( mol.atoms) - set( self.selected)
+        selected = set( mol.atoms) & set( self.selected)
         # we must check if the selection defines one connected subgraph of the molecule
         # otherwise the coordinate generation will not work
         if len( selected) == 1:
