@@ -33,7 +33,7 @@ import dom_extensions
 import math
 import operator
 import os_support
-import geometry
+from oasa import geometry
 
 from singleton_store import Screen
 
@@ -102,6 +102,10 @@ note that this is not an ODF (Open Document Format) export."""
         self.add_oval( o, page)
       elif o.object_type == 'polygon':
         self.add_polygon( o, page)
+      elif o.object_type == 'polyline':
+        self.add_polyline( o, page)
+      else:
+        pass
 
 #    dom_ext.safe_indent( root)
     
@@ -307,6 +311,14 @@ note that this is not an ODF (Open Document Format) export."""
     self.create_oo_polygon( points, page, style_name)
 
 
+  def add_polyline( self, o, page):
+    s = graphics_style( stroke_color=self.paper.any_color_to_rgb_string( o.line_color),
+                        stroke_width=Screen.px_to_cm( o.line_width))
+    style_name = self.get_appropriate_style_name( s)
+    points = [map( Screen.px_to_cm, p.get_xy()) for p in o.points]
+    self.create_oo_polyline( points, page, style_name)
+
+
   def add_rect( self, o, page):
     s = graphics_style( stroke_color=self.paper.any_color_to_rgb_string( o.line_color),
                         fill_color=self.paper.any_color_to_rgb_string( o.area_color),
@@ -619,8 +631,8 @@ note that this is not an ODF (Open Document Format) export."""
     w = self.paper.get_paper_property( 'size_x')/10.0
     h = self.paper.get_paper_property( 'size_y')/10.0
     s = dom_ext.elementUnder( root, 'office:styles')
-    as = dom_ext.elementUnder( root, 'office:automatic-styles')
-    pm = dom_ext.elementUnder( as, 'style:page-layout', (('style:name','PM1'),))
+    astyles = dom_ext.elementUnder( root, 'office:automatic-styles')
+    pm = dom_ext.elementUnder( astyles, 'style:page-layout', (('style:name','PM1'),))
     dom_ext.elementUnder( pm, 'style:page-layout-properties', (('fo:page-height','%fcm' % h),
                                                                ('fo:page-width','%fcm' % w),
                                                                ('style:print-orientation','portrait'),
@@ -628,8 +640,8 @@ note that this is not an ODF (Open Document Format) export."""
                                                                ('fo:margin-left','0.5cm'),
                                                                ('fo:margin-top','0.5cm'),
                                                                ('fo:margin-right','0.5cm')))
-    dp = dom_ext.elementUnder( as, 'style:style', (('style:family', 'drawing-page'),
-                                                   ('style:name', 'dp1')))
+    dp = dom_ext.elementUnder( astyles, 'style:style', (('style:family', 'drawing-page'),
+                                                        ('style:name', 'dp1')))
     dom_ext.elementUnder( dp, 'style:drawing-page-properties', (('draw:backgroud-size','border'),
                                                                 ('draw:fill', 'none')))
 
