@@ -33,7 +33,7 @@ import dom_extensions
 import math
 import operator
 import os_support
-import geometry
+from oasa import geometry
 
 from singleton_store import Screen
 
@@ -599,8 +599,8 @@ note that this is not an ODF (Open Document Format) export."""
     w = self.paper.get_paper_property( 'size_x')/10.0
     h = self.paper.get_paper_property( 'size_y')/10.0
     s = dom_ext.elementUnder( root, 'office:styles')
-    as = dom_ext.elementUnder( root, 'office:automatic-styles')
-    pm = dom_ext.elementUnder( as, 'style:page-master', (('style:name','PM1'),))
+    astyles = dom_ext.elementUnder( root, 'office:automatic-styles')
+    pm = dom_ext.elementUnder( astyles, 'style:page-master', (('style:name','PM1'),))
     dom_ext.elementUnder( pm, 'style:properties', (('fo:page-height','%fcm' % h),
                                                    ('fo:page-width','%fcm' % w),
                                                    ('style:print-orientation','portrait'),
@@ -608,8 +608,8 @@ note that this is not an ODF (Open Document Format) export."""
                                                    ('fo:margin-left','0.5cm'),
                                                    ('fo:margin-top','0.5cm'),
                                                    ('fo:margin-right','0.5cm')))
-    dp = dom_ext.elementUnder( as, 'style:style', (('style:family', 'drawing-page'),
-                                                   ('style:name', 'dp1')))
+    dp = dom_ext.elementUnder( astyles, 'style:style', (('style:family', 'drawing-page'),
+                                                        ('style:name', 'dp1')))
     dom_ext.elementUnder( dp, 'style:properties', (('draw:backgroud-size','border'),
                                                    ('draw:fill', 'none')))
 
