@@ -20,8 +20,8 @@
 
 import misc
 import marks
-from sets import Set
-import geometry
+
+from oasa import geometry
 from math import sin, cos, sqrt, pi
 import types
 import oasa
@@ -35,7 +35,6 @@ from tuning import Tuning
 import operator
 
 
-
 class vertex_common( object):
   """implements some properties and methods common for all vertices
   (children of oasa.chem_vertex), such as numbering and mark support"""
@@ -48,7 +47,7 @@ class vertex_common( object):
 
 
   def __init__( self):
-    self.marks = Set()
+    self.marks = set()
     # numbering
     self._show_number = True
     self._number = None
@@ -190,7 +189,7 @@ class vertex_common( object):
 
 
   def reposition_marks( self):
-    ms = Set( [m for m in self.marks if m.auto])
+    ms = set( [m for m in self.marks if m.auto])
     self.marks -= ms
     for m in ms:
       x, y = self.find_place_for_mark( m.__class__.__name__)
@@ -288,7 +287,7 @@ class drawable_chem_vertex( oasa.chem_vertex, meta_enabled, area_colored, point_
   all the methods are tuned for always shown texts; atoms need to override something"""
 
   # these values will be automaticaly read from paper.standard on __init__
-  meta__used_standard_values = ['line_color','area_color','font_size','font_family']
+  meta__used_standard_values = ['line_color','area_color','font_size','font_family','show_hydrogens']
   # undo meta infos
   meta__undo_fake = ('xml_ftext',)
   meta__undo_simple = ()
@@ -454,9 +453,9 @@ class drawable_chem_vertex( oasa.chem_vertex, meta_enabled, area_colored, point_
     if self.is_part_of_linear_fragment():
       self.pos = 'center-first'
       return 
-    as = self.get_neighbors()
+    atms = self.get_neighbors()
     p = 0
-    for a in as:
+    for a in atms:
       if a.x < self.x:
         p -= 1
       elif a.x > self.x:
@@ -619,7 +618,7 @@ class drawable_chem_vertex( oasa.chem_vertex, meta_enabled, area_colored, point_
     if self.ftext:
       self.ftext.delete()
     [m.delete() for m in self.marks]
-    self.marks = Set()
+    self.marks = set()
     return self
 
 
@@ -674,14 +673,14 @@ class drawable_chem_vertex( oasa.chem_vertex, meta_enabled, area_colored, point_
 
 
   def transform( self, tr):
-    x, y = tr.transform_xy( self.x, self.y)
+    x, y, z = tr.transform_xyz( self.x, self.y, self.z)
     self.move_to( x, y, dont_move_marks=1)
+    self.z = z
     for m in self.marks:
       m.transform( tr)
 
 
 
-
   def bbox( self, substract_font_descent=False):
     """returns the bounding box of the object as a list of [x1,y1,x2,y2]"""
     if self.item:
